 /*--------------------------------------------------------------
  * Copyright (C), 2010
  * SAS Institute Inc., Cary, NC, USA.  All Rights Reserved
  *
  * NAME:      rdui_envlist.sas
  * PURPOSE:   This will list out all of the environments and their
  *            name to a dataset.
  *
  * USAGE:     
  *            rdui_envlist(outputDS);  
  * PARAMETERS:
  *    THESE PARAMETERS ARE REQUIRED
  *    outputDS   :  The name that the output should be written to.
  * 
  *-------------------------------------------------------------*/




%macro rdui_envlist(outputDS);


data &outputDS;
set sashelp.vlibnam;
where libname not in ('SASHELP',
                      'MAPS', 
                      '_RISKCUR',
                      '_RISKCWD',
                      '_RSKJTMP',
                      '_RISKTMP',
                      '_CMPWORK',
                      '_RSKWORK') and
      libname ^like '_RKD%';
  length envname $ 256;
  length ENV_LIB $ 8;
  length ENV_NAME $32;
  length ENV_UP_NAME $32;
  length ENV_LIB_AND_NAME $41;
  length ENV_UP_LIB_AND_NAME $41;
  length IS_SAMPLE $1;
  length ENV_PATH $1024;
  length PATH_ORDER 8;
  retain ENV_NUM 0;


  keep ENV_LIB  ENV_NAME ENV_UP_NAME ENV_LIB_AND_NAME ENV_UP_LIB_AND_NAME IS_SAMPLE ENV_PATH PATH_ORDER ENV_NUM;

  ENV_LIB   = libname;
  IS_SAMPLE = '0';

  if libname eq 'SASHELP'  then do;
  end;
  else if libname eq 'MAPS' then do;
  end;
  else do;

  if libname eq 'RISKSAMP' then do;
     IS_SAMPLE='1';
  end;

  filrf='rd15224';                                                                                                                      

  /* Assigns the fileref of mydir to the directory and opens the directory */                                                                    
  rc=filename(filrf,path);           

  if ( rc = 0 ) and   (fileref('rd15224') = 0 ) then do;  

  libdid=dopen(filrf);                                                                                                      
  /* Check that we have permissions to this path and directory */

  if ( libdid ) then do;
 
  /* Returns the number of members in the directory */                                                                   
  libmemcnt=dnum(libdid); 

  /* Loops through entire directory */                                                                                                  
  do i = 1 to libmemcnt;  



    /* Returns the extension from each file */                                                                                                                                    
    envname=dread(libdid,i); 


    if indexc(envname,'.') = 0 then do;;

      ENV_NAME = envname;
      ENV_UP_NAME = upcase(ENV_NAME);
      ENV_LIB_AND_NAME =  trim(ENV_LIB) || '.' || trim(ENV_NAME);
      ENV_UP_LIB_AND_NAME = trim(ENV_LIB) || '.' || upcase(trim(ENV_NAME));


      envdirpath=trim(path)||'/'||trim(envname);

	  ENV_PATH = trim(envdirpath);
	  PATH_ORDER = i;


      envdirrf='rd15834';                                                                                                                      
                                                                                                                                        
      /* Assigns the fileref of mydir to the directory and opens the directory */                                                                    
      rc=filename(envdirrf,envdirpath);                                                                                                
      envdirdid=dopen(envdirrf);    

                
      if envdirdid > 0 then do; 

        /* Returns the number of members in the directory */                                                                   
        envdircnt=dnum(envdirdid); 

        /* Loops through entire directory */                                                                                                  
        do i2 = 1 to envdircnt;  
          
          filenm=dread(envdirdid,i2); 
          filenm=upcase(trim(filenm));

          if filenm eq "RD_ENVIRONMENT.SAS7BDAT" then do;
		     ENV_NUM = (ENV_NUM + 1);

             output;
          end;
        end;
        rc=dclose(envdirdid);  
            /** deassign */
        rc=filename(envdirrf);     
      end;

    end; /* else do */

   end; /* first do */


  /* Closes the directory */                                                                                                            
  rc=dclose(libdid);  

  end;

  /** deassign **/ 
  rc=filename(filrf);   

  end; 

  end;  /* parse out sashelp */

run;

proc sort data=&outputDS out=&outputDS;
by ENV_UP_LIB_AND_NAME PATH_ORDER;
run;

%mend rdui_envlist;
